/*
   FNT - Stream handler for socket connections
   reading data from socket

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/error.h"
#include "../include/stream.h"
#include "../include/mainloop.h"

#include "main.h"
#include "utils.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

/*** file scope type declarations **************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

DLLEXPORT bool
read_stream (fnt_stream_t * stream, gchar * buffer, gsize len,
	      gsize * bytes_read)
{
    GIOStatus status;
    GError *error = NULL;
    __fnt_driver_data_t *driver_data;

    if (stream->driver_data == NULL)
	return false;

    if (stream->state != FNT_STREAM_STATE_ok)
	return false;

    *bytes_read = 0;

    driver_data = (__fnt_driver_data_t *) stream->driver_data;

    stream->state = FNT_STREAM_STATE_reading;

    status =
	g_io_channel_read_chars (driver_data->socket, buffer, len,
				 bytes_read, &error);



    switch (status)
    {
    case G_IO_STATUS_NORMAL:
	stream->state = FNT_STREAM_STATE_ok;
	return true;
	break;
    case G_IO_STATUS_EOF:
	stream->state = FNT_STREAM_STATE_closing;
	if (!fnt_event_raise ("stream", "onClose", (void *) stream))
	    fnt_stream_close (stream);
	return false;
	break;
    }
/*
    printf ("\
G_IO_STATUS_ERROR = %d\n\
G_IO_STATUS_NORMAL = %d\n\
G_IO_STATUS_EOF = %d\n\
G_IO_STATUS_AGAIN = %d\n\n\
", G_IO_STATUS_ERROR, G_IO_STATUS_NORMAL, G_IO_STATUS_EOF, G_IO_STATUS_AGAIN);
    printf ("status = %d\n", status);
*/
    return false;

}
